/*                                                     -*- linux-c -*-
    Copyright (C) 2004 Olivier Doar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/


/* midiview version 0.01 - alsaseq midi event viewer
   2004, Olivier Doar

   Predominantly ripped from the specimen sources, which are...
   predominantly ripped from the alsa programming tutorial */

      
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <alsa/asoundlib.h>

static snd_seq_t *open_seq() 
{
     snd_seq_t *seq_handle;
     int portid;

     if (snd_seq_open(&seq_handle, "default", SND_SEQ_OPEN_INPUT, 0) < 0) {
	  fprintf(stderr, "Error opening ALSA sequencer.\n");
	  exit(1);
     }
     snd_seq_set_client_name(seq_handle, "Midiview");
     if ((portid = snd_seq_create_simple_port(seq_handle, "Midiview",
					      SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE,
					      SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
	  fprintf(stderr, "Error creating sequencer port.\n");
	  exit(1);
     }
     return(seq_handle);
}

static void midi_action(snd_seq_t *seq_handle) 
{
     snd_seq_event_t *ev;

     do {
	  snd_seq_event_input(seq_handle, &ev);
	  switch (ev->type) {
	  case SND_SEQ_EVENT_CONTROLLER: 
	       fprintf(stderr, "Control event  Channel %2d   Number   %3d     Value %3d       \n",
		       ev->data.control.channel, ev->data.control.param,ev->data.control.value);
	       break;
	  case SND_SEQ_EVENT_PITCHBEND:
	       fprintf(stderr, "Pitchbender    Channel %2d   Value %5d                         \n", 
		       ev->data.control.channel, ev->data.control.value);
	       break;
	  case SND_SEQ_EVENT_NOTEON:
	       fprintf(stderr, "Note on        Channel %2d   Velocity %3d     Note  %3d          \n", 
		       ev->data.note.channel, ev->data.note.velocity, ev->data.note.note);		/**< note */
	       break;        
	  }
	  snd_seq_free_event(ev);
     } while (snd_seq_event_input_pending(seq_handle, 0) > 0);
}

int main(int argc, char *argv[]) 
{
     snd_seq_t *seq_handle;
     int npfd;
     struct pollfd *pfd;

    seq_handle = open_seq();
     npfd = snd_seq_poll_descriptors_count(seq_handle, POLLIN);
     pfd = (struct pollfd *)alloca(npfd * sizeof(struct pollfd));
     snd_seq_poll_descriptors(seq_handle, pfd, npfd, POLLIN);

     while (1) {
	  if (poll(pfd, npfd, 100) > 0) {
	       midi_action(seq_handle);
	  }
     }
}
